/**
 * Un conto bancario.<br/>
 * Ha un saldo che pu essere modificato da depositi e prelievi.
 * Il conto  identificato da un numero di conto. 
 * Il controllo sulla univocit del numero di Conto non 
 * restonsabilit di ContoCorrente
 * Versione 2.0 
 */
public class ContoCorrente {
   private double saldo;
   private int numero;

   /** 
    * Costruisce un conto bancario con numero di conto 
    * uguale a 1 e saldo uguale a zero
    */
   public ContoCorrente() {
      this(1, 0);
   }

   /** 
    * Costruisce un conto bancario con saldo uguale a zero
    * @param numeroConto numero del conto
    */
   public ContoCorrente(int numeroConto) {
      this(numeroConto, 0);
   }

   /**
    * Costruisce un conto bancario con un saldo assegnato 
    * @param saldoIniziale il saldo iniziale
    */ 
   public ContoCorrente(int numeroConto, double saldoIniziale) {
      numero = numeroConto;
      saldo = saldoIniziale;
   }

   /**
    * Versa denaro nel conto bancario.
    * (post-condizione: getSaldo() >= 0 )
    * @param importo l'importo da versare
    * (pre-condizione: importo >= 0)
    */
   public void deposita(double importo) {
      assert importo >= 0 : "Deposita: importo deve essere positivo";
      double nuovoSaldo = saldo + importo;
      saldo = nuovoSaldo;
      assert validateState() : "Deposita: saldo non valido";
   }

   /**
    * Preleva denaro dal conto bancario.
    * (post-condizione: getSaldo() >= 0 )
    * @param importo l'importo da prelevare.
    * Deve essere inferiore al saldo disponibile.
    * (pre-condizione: importo >= 0 )
    */
   public void preleva(double importo) {
      assert importo >= 0 : "Preleva: importo deve essere positivo";
      double nuovoSaldo = saldo-importo;
      saldo = nuovoSaldo;
      assert validateState() : "Preleva: saldo non valido";
   }

   /**
    * Ispeziona il valore del saldo attuale del conto bancario
    * @return il saldo attuale
   */
   public double getSaldo() {
      return saldo;
   }
   
   /**
    * @return Il numero del conto corrente
    */
   public int getNumeroCC(){
      return numero; 
   }
   
   /** Controlla l'invariante (o gli invarianti) di classe.<br/>
    *  Per ora solo saldo>=0
    */
   private boolean validateState(){
      return getSaldo()>=0 ? true : false;
   }
   
   /** 
    * Restituisce una rappresentazione in forma di stringa
    * dello stato del conto corrente.
    * @return Descrizione del conto corrente.
    */
   @Override
   public String toString(){
      return "Conto numero "+ getNumeroCC()+" con saldo "+getSaldo(); 
   }

   /** 
    * Confronta questo conto con un altro
    * Due conti correnti sono uguali se hanno lo
    * stessi numero (e saldo).
    * @param o oggetto da confrontare (se non 
    * un conto sono diversi)
    * @return true se i conti sono uguali
    * @todo se i numeri di conto sono uguali e i saldi
    * diversi potrebbe essere un problema
    */
   @Override
   public boolean equals(Object o){
     if (o instanceof ContoCorrente){
        ContoCorrente cc = (ContoCorrente)o;
        return (cc.getNumeroCC() == this.getNumeroCC()
           && cc.getSaldo() == this.getSaldo());
     }
     return false;
   }
}

